#ifndef _CHRONOPTICS_TOF_STREAM_HPP_
#define _CHRONOPTICS_TOF_STREAM_HPP_

#include <chronoptics/tof/stream.h>

#include <chronoptics/tof/base.hpp>

namespace chronoptics {
namespace tof {

/** The different frame types that are supported
 */
enum class FrameType {
  RAW_COMMON = 0, /** The 16bits coming off the OPT8241 image sensor */
  COMMON = 1, /** Common (A + B) image, 4 or 12 bits depending on the sensor */
  RAW = 2, /** Raw (A - B) image 12 bits signed data */
  REAL = 3, /** The real image */
  IMAGINARY = 4, /** The imaginary image */
  PHASE = 5, /** The phase image */
  AMPLITUDE = 6, /** The amplitude image */
  RADIAL = 7, /** The radial distance image */
  INTENSITY = 8, /** The intensity image */
  X = 9, /** The x image of the point cloud */
  Y = 10, /** The y image of the point cloud */
  Z = 11, /** The z image of the point cloud */
  XYZ = 12, /** A complete pointcloud image: float x, float y, float z, float padding */
  XYZ_AMP = 13, /** A complete pointcloud image with amplitude: float x, float y, float z, float amplitude */
  XYZ_BGR = 14, /** A complete pointcloud image with bgr: float x, float y, float z, uint8 b, uint8 g, uint8 r, uint8 padding */
  XYZ_BGR_I = 15, /** A complete pointcloud image with bgr and intensity: float x, float y, float z, uint8 b, uint8 g, uint8 r, uint8 intensity */
  BGR = 18, /** A color image, encode bgr */
  YUV = 19, /** A color image encoded in YUV422 */
  RAW_AB = 20, /** A raw image containing both the A and B tap */
  MJPEG = 21, /** a color image encoded in mjpeg */
  BGR_PROJECTED = 22, /** A color image projected to the XYZ locations of the depth sensor */
  COMPRESSED_DOT = 23, /** The averaged value for each dot. In the XYZAmp format. */
};

/** This class describes the different data streams coming from the camera
*/
class Stream : public detail::Base<tof_stream, tof_stream_delete> {
 public:
  /** Construct from pointer */
  Stream(tof_stream_t ptr = nullptr) {
    this->ptr_ = ptr;
  }

  /** Get the frame type
   * @return The frame type
   */
  FrameType frame_type() const {
    return static_cast<FrameType>(tof_stream_frame_type(this->ptr_, TOF_ERROR_HANDLER{}));
  }

  /** Get the frame id
   * @return The frame id
   */
  uint32_t frame_id() const {
    return tof_stream_frame_id(this->ptr_, TOF_ERROR_HANDLER{});
  }

  /** Get the modulation frequency of the frame
   * @return The modulation frequency
   */
  float modulation_frequency() const {
    return tof_stream_modulation_frequency(this->ptr_, TOF_ERROR_HANDLER{});
  }

  /** Get the integration time of the frame
   * @return The integration time
   */
  uint32_t integration_time() const {
    return tof_stream_integration_time(this->ptr_, TOF_ERROR_HANDLER{});
  }

  /** Get a bit array of the processing performed on the frame
   * @return A bit array
   */
  uint32_t process() const {
    return tof_stream_process(this->ptr_, TOF_ERROR_HANDLER{});
  }

};

/** Get the string representation of the frame type
 * @param frame_type Frame type
 * @return String representation of frame type
 */
inline const char* frame_type_to_string(FrameType frame_type) {
  return tof_frame_type_to_string(static_cast<tof_frame_type>(frame_type), TOF_ERROR_HANDLER{});
}

} // tof
} // chronoptics

#endif
